import time
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.text import get_valid_filename, slugify
from django.utils.encoding import python_2_unicode_compatible


EXTENSION_CHOICES = (
    ('.ini', _('INI file')),
    ('.yml', _('YAML file')),
    ('.xml', _('XML file')),
    ('.json', _('JSON file')),
)


@python_2_unicode_compatible
class Vassal(models.Model):
    """
    Model for uWSGI Vassals
    """
    title = models.CharField(
        _('Name'),
        max_length=250
    )
    name = models.CharField(
        _('Filename'),
        help_text=_('Name of uwsgi config file'),
        max_length=255,
        db_column='name',
        editable=False
    )
    extension = models.CharField(
        _('Extension'),
        max_length=4,
        default=EXTENSION_CHOICES[0],
        choices=EXTENSION_CHOICES
    )
    config = models.TextField(
        _('Config'),
        help_text=_('Config file blob'),
        db_column='config'
    )
    ts = models.FloatField(
        _('Unix Timestamp'),
        help_text=_('A number representing the modification time of this row in UNIX format'),
        db_column='ts',
        editable=False
    )
    uid = models.PositiveSmallIntegerField(
        _('The UID of the vassal instance'),
        help_text=_('Required in Tyrant mode (secure multi-user hosting) mode only.'),
        db_column='uid',
        blank=True,
        null=True
    )
    gid = models.PositiveSmallIntegerField(
        _('The GID of the vassal instance'),
        help_text=_('Required in Tyrant mode (secure multi-user hosting) mode only.'),
        db_column='gid',
        blank=True,
        null=True
    )
    created = models.DateTimeField(
        _('Created at'),
        auto_now_add=True
    )
    updated = models.DateTimeField(
        _('Updated'),
        auto_now=True,
        blank=True,
        null=True
    )
    enabled = models.BooleanField(
        _('Enabled'),
        default=True
    )


    def __str__(self):
        return self.title


    def save(self, *args, **kwargs):
        self.name = get_valid_filename(slugify(self.title) + self.extension)
        self.ts = time.time()
        super(Vassal, self).save(*args, **kwargs)


    class Meta:
        verbose_name = _("Emperor's Vassal")
        verbose_name_plural = _("Emperor's Vassals")
        db_table = 'vassals'



# -*- coding: utf-8 -*-

import sys
import os
on_rtd = os.environ.get('READTHEDOCS', None) == 'True'

if not on_rtd:
    import sphinx_rtd_theme
    html_theme = 'sphinx_rtd_theme'
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

sys.path.insert(0, os.path.abspath('..'))
from django_uwsgi import __version__


extensions = [
    'sphinx.ext.intersphinx',
    'releases',
]

releases_issue_uri = "https://edugit.org/AlekSIS/libs/django-uwsgi-ng/-/issues/%s"
releases_release_uri = "https://edugit.org/AlekSIS/libs/django-uwsgi-ng/-/tree/%s"

templates_path = ['_templates']

source_suffix = '.rst'

master_doc = 'index'

project = u'django-uwsgi-ng'
copyright = u'2016, Eugene MechanisM'

version = __version__
release = __version__

exclude_patterns = ['_build']

pygments_style = 'sphinx'

html_static_path = ['_static']

htmlhelp_basename = 'django-uwsgidoc'

latex_elements = {

}

latex_documents = [
    ('index', 'django-uwsgi-ng.tex', u'django-uwsgi-ng Documentation',
     u'Eugene MechanisM', 'manual'),
]

man_pages = [
    ('index', 'django-uwsgi-ng', u'django-uwsgi-ng Documentation',
     [u'Eugene MechanisM'], 1)
]

texinfo_documents = [
    ('index', 'django-uwsgi-ng', u'django-uwsgi-ng Documentation',
     u'Eugene MechanisM', 'django-uwsgi-ng', 'One line description of project.',
     'Miscellaneous'),
]
intersphinx_mapping = {'uwsgi': ('http://uwsgi-docs.readthedocs.org/en/latest/', None)}



from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand
from django.conf import settings
from django.apps import apps
import os
import sys
import multiprocessing
from tempfile import TemporaryDirectory

root = os.getcwd()
django_project = os.path.basename(root)


class Command(BaseCommand):
    help = "Runs this project as a uWSGI application. Requires the uwsgi binary in system path."

    def add_arguments(self, parser):
        parser.add_argument('uwsgi_options', nargs='*')

    def handle(self, *args, **options):
        os.environ.setdefault('DJANGO_SETTINGS_MODULE', '%s.settings' % django_project)

        # Define hard defaults
        UWSGI_OPTIONS = {
            "module": '%s.wsgi' % django_project,
            "http-socket": ":8000",
            "http-auto-chunked": True,
            "http-keepalive": 75,
            "http-timeout": 75,
            "no-defer-accept": True,
            "workers": int(multiprocessing.cpu_count() * 1.5),
            "auto-procname": True,
            "procname-prefix-spaced": "[uWSGI %s]" % django_project,
            "vacuum": True,
            "master": True,
            "virtualenv": sys.prefix,
            "pp": root,
            "post-buffering": 1048576,
            "buffer-size": 65535,
            "reload-on-rss": 300,
            "enable-threads": True,
            "lazy-apps": False,
            "thunder-lock": True,
            "autoload": True,
            "no-orphans": True,
            "memory-report": True,
            "disable-logging": True,
            "ignore-sigpipe": True,
            "ignore-write-errors": True,
            "disable-write-exception": True,
            "harakiri": 60,
            "harakiri-verbose": True,
            "cache2": "name=%s,items=20000,keysize=128,blocksize=4096" % django_project,
        }

        # Defien configuration-dependent defaults
        if UWSGI_OPTIONS.get("workers", 1) > 1 and not "cheaper" in UWSGI_OPTIONS:
            UWSGI_OPTIONS["cheaper"] = int(UWSGI_OPTIONS["workers"] / 1.5)
        if getattr(settings, "UWSGI_SERVE_STATIC", settings.DEBUG):
            UWSGI_OPTIONS.setdefault("static-map", []).append('%s=%s' % (settings.STATIC_URL, settings.STATIC_ROOT))
        if getattr(settings, "UWSGI_SERVE_MEDIA", settings.DEBUG) and hasattr(settings, "MEDIA_ROOT"):
            UWSGI_OPTIONS.setdefault("static-map", []).append('%s=%s' % (settings.MEDIA_URL, settings.MEDIA_ROOT))
        if settings.DEBUG:
            UWSGI_OPTIONS["py-autoreload"] = 2
        if 'django_uwsgi' in settings.EMAIL_BACKEND:
            UWSGI_OPTIONS["spooler"] = "/tmp"
            UWSGI_OPTIONS["spooler-import"] = "django_uwsgi.tasks"

        # Drop defaults that were set as environment variables
        for key in UWSGI_OPTIONS:
            if "UWSGI_%s" % key.upper().replace("-", "_") in os.environ:
                del UWSGI_OPTIONS[key]

        # Merge uWSGI options from settings
        opts = getattr(settings, "UWSGI_OPTIONS", {})
        legacy_opts = getattr(settings, "UWSGI", {})
        UWSGI_OPTIONS.update(legacy_opts)
        UWSGI_OPTIONS.update(opts)

        if apps.ready:
            argv = ['uwsgi']
            # Pass the config dict as INI file
            with TemporaryDirectory() as tmpdir:
                with open(os.path.join(tmpdir, "uwsgi.ini"), "w") as ini_out:
                    print("[uwsgi]", file=ini_out)
                    for option, values in UWSGI_OPTIONS.items():
                        if not isinstance(values, (list, tuple)):
                            values = [values]
                        for value in values:
                            if isinstance(value, bool):
                                value = "1" if value else "0"
                            elif not isinstance(value, str):
                                value = str(value)
                            if "\n" in value:
                                raise ImproperlyConfigured("Line breaks are not allowed in uWSGI options.")
                            print("%s = %s" % (option, value), file=ini_out)
                argv += ['--ini', os.path.join(tmpdir, "uwsgi.ini")]
                argv += options['uwsgi_options']
                os.execvp('uwsgi', argv)

    def usage(self, subcomand):
        return r"""
  run this project on the uWSGI server
  All command-line arguments are passed verbatim to the uwsgi command
        """



from django.conf.urls import url
from django.contrib import admin
from . import views

urlpatterns = [
    url(r'^$', admin.site.admin_view(views.UwsgiStatus.as_view()),
        name='uwsgi_index'),
    url(r'^reload/$', admin.site.admin_view(views.UwsgiReload.as_view()),
        name='uwsgi_reload'),
    url(r'^cache_clear/$', admin.site.admin_view(
        views.UwsgiCacheClear.as_view()), name='uwsgi_cache_clear'),
    url(r'^log/$', admin.site.admin_view(views.UwsgiLog.as_view()),
        name='uwsgi_log'),
    url(r'^signal/$', admin.site.admin_view(views.UwsgiSignal.as_view()),
        name='uwsgi_signal'),
]



from __future__ import unicode_literals
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
try:
    from django.urls import reverse_lazy
except ImportError:
    from django.core.urlresolvers import reverse_lazy
from django.views.generic import View, TemplateView

from django.core.exceptions import PermissionDenied
from django.apps import apps
from . import uwsgi


class UwsgiStatus(TemplateView):
    '''
    uWSGI Status View
    '''
    if apps.is_installed('wagtail.admin'):
        template_name = 'uwsgi/wagtail_uwsgi.html'
    else:
        template_name = 'uwsgi/uwsgi.html'

    def get (self, request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied
            
        return super(UwsgiStatus, self).get(request, *args, **kwargs)
        
    def get_context_data(self, **kwargs):
        context = super(UwsgiStatus, self).get_context_data(**kwargs)
        if uwsgi is None:
            context['unavailable'] = True
            return context
        else:
            from .stats import get_uwsgi_stats
            context.update(get_uwsgi_stats())
            return context


class UwsgiCacheClear(View):
    '''
    Clear uWSGI Cache View
    '''
    def get(self, request):
        if not request.user.is_superuser:
            raise PermissionDenied
        if uwsgi is not None and uwsgi.masterpid() > 0:
            uwsgi.cache_clear()
            messages.add_message(request, messages.SUCCESS, _(
                'uWSGI Cache cleared!'), fail_silently=True)
        else:
            messages.add_message(request, messages.ERROR, _(
                'The uWSGI master process is not active'), fail_silently=True)
        return HttpResponseRedirect(reverse_lazy('uwsgi_index'))


class UwsgiReload(View):
    '''
    Reload uWSGI View
    '''
    def get(self, request):
        if not request.user.is_superuser:
            raise PermissionDenied
        if uwsgi is not None and uwsgi.masterpid() > 0:
            uwsgi.reload()
            messages.add_message(
                request, messages.SUCCESS, _('uWSGI reloaded!'), fail_silently=True)
        else:
            messages.add_message(request, messages.ERROR, _(
                'The uWSGI master process is not active'), fail_silently=True)
        return HttpResponseRedirect(reverse_lazy('uwsgi_index'))


class UwsgiLog(View):
    '''
    uWSGI Log View
    '''
    def post(self, request):
        if not request.user.is_superuser:
            raise PermissionDenied
        if uwsgi is not None:
            uwsgi.log(request.POST.get('log_message'))
            messages.add_message(request, messages.SUCCESS, _(
                'uWSGI Log message has been sent!'), fail_silently=True)
        else:
            messages.add_message(request, messages.ERROR, _(
                'uWSGI is not available!'), fail_silently=True)
        return HttpResponseRedirect(reverse_lazy('uwsgi_index'))


class UwsgiSignal(View):
    '''
    uWSGI Signal View
    '''
    def post(self, request):
        if not request.user.is_superuser:
            raise PermissionDenied
        if uwsgi is not None:
            uwsgi.signal(int(request.POST.get('signal_number')))
            messages.add_message(request, messages.SUCCESS, _(
                'uWSGI signal has been sent!'), fail_silently=True)
        else:
            messages.add_message(request, messages.ERROR, _(
                'uWSGI is not available!'), fail_silently=True)
        return HttpResponseRedirect(reverse_lazy('uwsgi_index'))



from debug_toolbar.panels import Panel
from django.utils.translation import ugettext_lazy as _
from . import uwsgi


class UwsgiPanel(Panel):
    '''
    uWSGI Debug Toolbar Panel
    '''
    title = _('uWSGI Status')
    nav_title = _('uWSGI Status')
    template = 'uwsgi/panel.html'

    @property
    def nav_subtitle(self):
        if uwsgi is not None:
            status = _('Version %s, %d Workers') % (
                str(uwsgi.version.decode()), int(uwsgi.numproc))
        else:
            status = _('uWSGI is missing =(')
        return status

    def generate_stats(self, request, response):
        if uwsgi is None:
            self.record_stats({'unavailable': True})
        else:
            from .stats import get_uwsgi_stats
            self.record_stats(get_uwsgi_stats())
    process_response = generate_stats



from wagtail.contrib.modeladmin.options import (
    ModelAdmin,
    modeladmin_register
)
from .models import Vassal
from .admin import VassalFields


class VassalModelAdmin(VassalFields, ModelAdmin):
    model = Vassal
    menu_icon = 'cogs'
    menu_order = 800
    add_to_settings_menu = True

modeladmin_register(VassalModelAdmin)



from __future__ import unicode_literals
from django.core.mail import get_connection
from django.conf import settings

from .decorators import spool
from . import pickle

if 'django_uwsgi' in settings.EMAIL_BACKEND:
    BACKEND = getattr(settings, 'UWSGI_EMAIL_BACKEND',
                      'django.core.mail.backends.smtp.EmailBackend')

    @spool
    def send_mail(arguments):
        conn = get_connection(backend=BACKEND)
        conn.send_messages([pickle.loads(arguments['body'])])



from django.template.base import TemplateDoesNotExist
from django.template.loader import BaseLoader
from django.conf import settings
from . import uwsgi


class Loader(BaseLoader):
    """uWSGI Template Loader"""

    is_usable = uwsgi

    def symbolize(self, name):
        return name.replace('.', '_').replace('/', '_').replace('-', '_')

    def load_template_source(self, template_name, template_dirs=None):
        filename = 'templates_' + self.symbolize(template_name)
        for app in settings.INSTALLED_APPS:
            try:
                symbol = '%s_%s' % (self.symbolize(app), filename)
                return (uwsgi.embedded_data(symbol), 'sym://%s' % symbol)
            except:
                pass
        raise TemplateDoesNotExist(template_name)



__version__ = '1.1.2'


try:
    import uwsgi
except ImportError:
    uwsgi = None

try:
    from six.moves import cPickle as pickle
except ImportError:
    import pickle

default_app_config = 'django_uwsgi.apps.DjangoUwsgiConfig'



import os

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

from django_uwsgi import __version__


os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name='django-uwsgi-ng',
    version=__version__,
    description='uWSGI stuff for Django projects',
    long_description=open('README.rst').read(),
    url='https://edugit.org/AlekSIS/libs/django-uwsgi-ng',
    author='Eugene MechanisM',
    author_email='eugene@mechanism.pro',
    maintainer='Dominik George',
    maintainer_email='dominik.george@teckids.org',
    license='MIT',
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Code Generators",
        'Topic :: Communications :: Email',
        'Framework :: Django',
    ],
    keywords='uwsgi, django, mail, cache, template',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    extras_require={
        'uwsgi': ['uwsgi'],
    },
)



try:
    from django.utils.encoding import force_bytes as stringify
except ImportError:
    from django.utils.encoding import smart_str as stringify

try:
    from django.core.cache.backends.base import DEFAULT_TIMEOUT
except IndexError:
    DEFAULT_TIMEOUT = object()

from django.core.cache.backends.base import BaseCache, InvalidCacheBackendError
from django.conf import settings


try:
    from . import uwsgi, pickle
except ImportError:
    if getattr(settings, "UWSGI_CACHE_FALLBACK", True):
        uwsgi = None
    else:
        raise InvalidCacheBackendError(
            "You're not running under uWSGI ! "
            "Set UWSGI_CACHE_FALLBACK=True in settings if you want to fallback "
            "to LocMemCache."
        )


if uwsgi:
    class UwsgiCache(BaseCache):
        """uWSGI cache backend"""
        def __init__(self, server, params):
            BaseCache.__init__(self, params)
            self._cache = uwsgi
            self._server = server

        def exists(self, key):
            return self._cache.cache_exists(stringify(key), self._server)

        def add(self, key, value, timeout=True, version=None):
            full_key = self.make_key(key, version=version)
            if self.exists(full_key):
                return False
            self._set(full_key, value, timeout)
            return True

        def get(self, key, default=None, version=None):
            full_key = self.make_key(key, version=version)
            val = self._cache.cache_get(stringify(full_key), self._server)
            if val is None:
                return default
            val = stringify(val)
            return pickle.loads(val)

        def _set(self, full_key, value, timeout):
            if timeout is True or timeout is DEFAULT_TIMEOUT:
                uwsgi_timeout = self.default_timeout
            elif timeout is None or timeout is False:
                # Django 1.6+: Explicitly passing in timeout=None will set a non-expiring timeout.
                uwsgi_timeout = 0
            elif timeout == 0:
                # Django 1.6+: Passing in timeout=0 will set-and-expire-immediately the value.
                uwsgi_timeout = -1
            else:
                uwsgi_timeout = timeout
            self._cache.cache_update(stringify(full_key), pickle.dumps(value), uwsgi_timeout, self._server)

        def set(self, key, value, timeout=True, version=None):
            full_key = self.make_key(key, version=version)
            self._set(full_key, value, timeout)

        def delete(self, key, version=None):
            full_key = self.make_key(key, version=version)
            self._cache.cache_del(stringify(full_key), self._server)

        def close(self, **kwargs):
            pass

        def clear(self):
            self._cache.cache_clear(self._server)
else:
    from django.core.cache.backends.locmem import LocMemCache as UwsgiCache
